@@ -1,6 +1,6 @@
 /*
 
-   Copyright 2005 Paul Eggleton & Holger Hans Peter Freyther
+   Copyright 2008 Paul Eggleton & Holger Hans Peter Freyther
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -44,11 +44,9 @@ void sync_cancelled(void)
 	/*user_cancelled_sync = TRUE;*/
 }
 
-static osync_bool opie_sync_settings_parse(OpiePluginEnv *env, const char *config, OSyncError **error)
+static osync_bool opie_sync_read_config(OpiePluginEnv *env, OSyncPluginInfo *info, OSyncError **error)
 {
-	osync_trace(TRACE_ENTRY, "%s(%p, %s, %p)", __func__, env, config, error);
-	xmlDoc *doc = NULL;
-	xmlNode *cur = NULL;
+	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, env, info, error);
 
 	/* Set defaults */
 	env->username = g_strdup("root");
@@ -62,94 +60,97 @@ static osync_bool opie_sync_settings_parse(OpiePluginEnv *env, const char *confi
 	env->localdir = g_strdup("/tmp");
 	env->notes_type = NOTES_TYPE_BASIC;
 
-	doc = xmlParseMemory(config, strlen(config));
-
-	if (!doc) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Unable to parse settings");
+	OSyncPluginConfig *config = osync_plugin_info_get_config(info);
+	if (!config) {
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Unable to get configuration");
 		goto error;
 	}
 
-	cur = xmlDocGetRootElement(doc);
-
-	if (!cur) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Unable to get configuration XML root element");
-		goto error_free_doc;
+	OSyncPluginConnection *conn = osync_plugin_config_get_connection(config);
+	if(conn) {
+		const char *addr = osync_plugin_connection_net_get_address(conn);
+		if(addr) {
+			g_free(env->host);
+			env->host = g_strdup(addr);
+		}
+		unsigned int port = osync_plugin_connection_net_get_port(conn);
+		if(port > 0)
+			env->device_port = port;
 	}
 
-	if (xmlStrcmp(cur->name, (xmlChar*)"config")) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Configuration file root node name is invalid");
-		goto error_free_doc;
+	OSyncPluginAuthentication *auth = osync_plugin_config_get_authentication(config);
+	if(auth) {
+		const char *username = osync_plugin_authentication_get_username(auth);
+		if(username) {
+			g_free(env->username);
+			env->username = g_strdup(username);
+		}
+		const char *password = osync_plugin_authentication_get_password(auth);
+		if(password) {
+			g_free(env->password);
+			env->password = g_strdup(password);
+		}
 	}
+	
+	OSyncList *advoptions = osync_plugin_config_get_advancedoptions(config);
+	for (; advoptions; advoptions = advoptions->next) {
+		OSyncPluginAdvancedOption *option = advoptions->data;
 
-	cur = cur->xmlChildrenNode;
+		const char *val = osync_plugin_advancedoption_get_value(option);
+		const char *name = osync_plugin_advancedoption_get_name(option);
+		g_assert(name);
+		g_assert(val);
+		const char *key = NULL;
 
-	while (cur != NULL) {
-		if(cur->type == XML_ELEMENT_NODE) {
-			char *str = (char *)xmlNodeGetContent(cur);
-			if (str) {
-				if (!xmlStrcmp(cur->name, (const xmlChar *)"username")) {
-					g_free(env->username);
-					env->username = g_strdup(str);
-				} else if (!xmlStrcmp(cur->name, (const xmlChar *)"password")) {
-					g_free(env->password);
-					env->password = g_strdup(str);
-				} else if (!xmlStrcmp(cur->name, (const xmlChar *)"hostname")) {
-					g_free(env->host);
-					env->host = g_strdup(str);
-				} else if (!xmlStrcmp(cur->name, (const xmlChar *)"url")) {
-					osync_trace(TRACE_INTERNAL, "The 'url' configuration parameter is deprecated - please use 'hostname' instead");
-					g_free(env->host);
-					env->host = g_strdup(str);
-				} else if (!xmlStrcmp(cur->name, (const xmlChar *)"port")) {
-					env->device_port = atoi(str);
-				} else if (!xmlStrcmp(cur->name, (const xmlChar *)"device")) {
-					if (!strcasecmp(str, "qtopia2"))
-						env->device_type = OPIE_SYNC_QTOPIA_2;
-					else
-						env->device_type = OPIE_SYNC_OPIE;
-				} else if (!xmlStrcmp(cur->name, (const xmlChar *)"conntype")) {
-					if (!strcasecmp(str, "scp"))
-						env->conn_type = OPIE_CONN_SCP;
-					else if ( strcasecmp(str, "none") == 0 )
-						env->conn_type = OPIE_CONN_NONE;
-					else
-						env->conn_type = OPIE_CONN_FTP;
-				} else if (!xmlStrcmp(cur->name, (const xmlChar *)"use_qcop")) {
-					if ( strcasecmp(str, "false") == 0 )
-						env->use_qcop = FALSE;
-					else 
-						env->use_qcop = TRUE;
-				} else if (!xmlStrcmp(cur->name, (const xmlChar *)"backupdir")) {
-					if(strlen(str) > 0)
-						env->backupdir = g_strdup(str);
-				} else if (!xmlStrcmp(cur->name, (const xmlChar *)"localdir")) {
-					g_free(env->localdir);
-					env->localdir = g_strdup(str);
-				} else if (!xmlStrcmp(cur->name, (const xmlChar *)"notestype")) {
-					if ( (!strcasecmp(str, "opie-notes")) || (!strcasecmp(str, "opie_notes")) )
-						env->notes_type = NOTES_TYPE_OPIE_NOTES;
-					else if ( strcasecmp(str, "basic") == 0 )
-						env->notes_type = NOTES_TYPE_BASIC;
-					else {
-						osync_error_set(error, OSYNC_ERROR_GENERIC, "Invalid value \"%s\" for configuration option \"%s\"", str, cur->name);
-						goto error_free_doc;
-					}
-				} else {
-					osync_error_set(error, OSYNC_ERROR_GENERIC, "Invalid configuration file option \"%s\"", cur->name);
-					goto error_free_doc;
-				}
-				xmlFree(str);
+		if (!strcmp(name, "ClientType")) {
+			if (!strcasecmp(val, "qtopia2"))
+				env->device_type = OPIE_SYNC_QTOPIA_2;
+			else
+				env->device_type = OPIE_SYNC_OPIE;
+		} 
+		else if (!strcmp(name, "ConnType")) {
+			if (!strcasecmp(val, "scp"))
+				env->conn_type = OPIE_CONN_SCP;
+			else if ( strcasecmp(val, "none") == 0 )
+				env->conn_type = OPIE_CONN_NONE;
+			else
+				env->conn_type = OPIE_CONN_FTP;
+		} 
+		else if (!strcmp(name, "NotesType")) {
+			if ( (!strcasecmp(val, "opie-notes")) || (!strcasecmp(val, "opie_notes")) )
+				env->notes_type = NOTES_TYPE_OPIE_NOTES;
+			else if ( strcasecmp(val, "basic") == 0 )
+				env->notes_type = NOTES_TYPE_BASIC;
+			else {
+				osync_error_set(error, OSYNC_ERROR_GENERIC, "Invalid value \"%s\" for configuration option \"%s\"", val, name);
+				goto error;
+			}
+		} 
+		else if (!strcmp(name, "BackupDir")) {
+			if(strlen(val) > 0)
+				env->backupdir = g_strdup(val);
+		} 
+		else if (!strcmp(name, "LocalDir")) {
+			if(strlen(val) > 0) {
+				g_free(env->localdir);
+				env->localdir = g_strdup(val);
 			}
+		} 
+		else if (!strcmp(name, "DisableQcop")) {
+			if(atoi(val))
+				env->use_qcop = FALSE;
+			else
+				env->use_qcop = TRUE;
+		} 
+		else {
+			osync_error_set(error, OSYNC_ERROR_GENERIC, "Invalid configuration file advanced option \"%s\"", name);
+			goto error;
 		}
-		cur = cur->next;
 	}
 
-	xmlFreeDoc(doc);
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
-	
-error_free_doc:
-	xmlFreeDoc(doc);
+
 error:
 	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
 	return FALSE;
@@ -294,6 +295,8 @@ static void connect(void *userdata, OSyncPluginInfo *info, OSyncContext *ctx)
 	g_free(tablepath);
 	if (!env->hashtable)
 		goto error;
+	if(!osync_hashtable_load(env->hashtable, &error))
+		goto error;
 
 	if(env->objtype == OPIE_OBJECT_TYPE_NOTE) {
 		/* Check if the notestype config option has changed since the last sync */
@@ -325,99 +328,99 @@ static void get_changes(void *userdata, OSyncPluginInfo *info, OSyncContext *ctx
 
 	if (osync_objtype_sink_get_slowsync(sink)) {
 		osync_trace(TRACE_INTERNAL, "Slow sync requested");
-		osync_hashtable_reset(env->hashtable);
+
+		if (!osync_hashtable_slowsync(env->hashtable, &error)) {
+			osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&error));
+			osync_context_report_osyncerror(ctx, error);
+			return;
+		}
 	}
 
 	xmlNode *item_node = opie_xml_get_first(env->doc, env->listelement, env->itemelement);
 	while(item_node)
 	{
-		char *uid = opie_xml_get_tagged_uid(item_node);
-		unsigned char *hash = hash_xml_node(env->doc, item_node);
-		
-		/* Convert category IDs to names that other systems can use */
-		g_mutex_lock(env->plugin_env->plugin_mutex);
-		char *categories_bkup = opie_xml_get_categories(item_node);
-		if(env->plugin_env->categories_doc && categories_bkup)
-			opie_xml_category_ids_to_names(env->plugin_env->categories_doc, item_node);
-		g_mutex_unlock(env->plugin_env->plugin_mutex);
-		
-		char *data = xml_node_to_text(env->doc, item_node);
-
-		/* Restore old categories value as we don't want to save this back to our XML file */
-		if(categories_bkup) {
-			opie_xml_set_categories(item_node, categories_bkup);
-			g_free(categories_bkup);
+		/* Create the new change to report */
+		OSyncChange *change = osync_change_new(&error);
+		if (!change) {
+			osync_context_report_osyncwarning(ctx, error);
+			osync_error_unref(&error);
 		}
-		
-		// Report every entry .. every unreported entry got deleted.
-		osync_hashtable_report(env->hashtable, uid);
+		else {
 
-		OSyncChangeType changetype = osync_hashtable_get_changetype(env->hashtable, uid, hash);
-
-		if (changetype != OSYNC_CHANGE_TYPE_UNMODIFIED) {
-			//Set the hash of the object (optional, only required if you use hashtabled)
-			osync_hashtable_update_hash(env->hashtable, changetype, uid, hash);
+			/* Retrieve and set the uid of the object */
+			char *uid = opie_xml_get_tagged_uid(item_node);
+			osync_change_set_uid(change, uid);
+			g_free(uid);
+			
+			/* Calculate and set the hash of the object */
+			unsigned char *hash = hash_xml_node(env->doc, item_node);
+			osync_change_set_hash(change, hash);
+			g_free(hash);
+			
+			/* Detect and set the type of change */
+			OSyncChangeType changetype = osync_hashtable_get_changetype(env->hashtable, change);
+			osync_change_set_changetype(change, changetype);
+			
+			/* Update the hashtable with the hash of the object */
+			osync_hashtable_update_change(env->hashtable, change);
 	
-			//Make the new change to report
-			OSyncChange *change = osync_change_new(&error);
-			if (!change) {
-				osync_context_report_osyncwarning(ctx, error);
-				osync_error_unref(&error);
-			}
-			else {
-	
-				//Now set the uid of the object
-				osync_change_set_uid(change, uid);
-				osync_change_set_hash(change, hash);
-				osync_change_set_changetype(change, changetype);
+			if (changetype != OSYNC_CHANGE_TYPE_UNMODIFIED) {
+				/* Convert category IDs to names that other systems can use */
+				g_mutex_lock(env->plugin_env->plugin_mutex);
+				char *categories_bkup = opie_xml_get_categories(item_node);
+				if(env->plugin_env->categories_doc && categories_bkup)
+					opie_xml_category_ids_to_names(env->plugin_env->categories_doc, item_node);
+				g_mutex_unlock(env->plugin_env->plugin_mutex);
+				
+				char *data = xml_node_to_text(env->doc, item_node);
 		
+				/* Restore old categories value as we don't want to save this back to our XML file */
+				if(categories_bkup) {
+					opie_xml_set_categories(item_node, categories_bkup);
+					g_free(categories_bkup);
+				}
+				
 				OSyncData *odata = osync_data_new(data, strlen(data) + 1, env->objformat, &error);
 				if (!odata) {
-					osync_change_unref(change);
 					osync_context_report_osyncwarning(ctx, error);
 					osync_error_unref(&error);
 				}
 				else {
 					osync_data_set_objtype(odata, osync_objtype_sink_get_name(sink));
 			
-					//Now you can set the data for the object
+					/* Set the data for the object */
 					osync_change_set_data(change, odata);
 					osync_data_unref(odata);
 			
-					// just report the change via
+					/* Report the change */
 					osync_context_report_change(ctx, change);
-			
-					osync_change_unref(change);
 				}
+				g_free(data);
 			}
+			
+			osync_change_unref(change);
 		}
-
-		g_free(data);
-		g_free(hash);
-		g_free(uid);
 		
 		item_node = opie_xml_get_next(item_node);
 	}
 
-	//When you are done looping and if you are using hashtables
-	//check for deleted entries ... via hashtable
-	int i;
-	char **uids = osync_hashtable_get_deleted(env->hashtable);
-	for (i=0; uids[i]; i++) {
+	/* Check for deleted entries using the hashtable */
+	OSyncList *u, *uids = osync_hashtable_get_deleted(env->hashtable);
+	for (u = uids; u; u = u->next) {
+		const char *uid = u->data;
+		
 		OSyncChange *change = osync_change_new(&error);
 		if (!change) {
-			g_free(uids[i]);
 			osync_context_report_osyncwarning(ctx, error);
 			osync_error_unref(&error);
 			continue;
 		}
 
-		osync_change_set_uid(change, uids[i]);
+		osync_change_set_uid(change, uid);
 		osync_change_set_changetype(change, OSYNC_CHANGE_TYPE_DELETED);
 
 		OSyncData *odata = osync_data_new(NULL, 0, env->objformat, &error);
 		if (!odata) {
-			g_free(uids[i]);
 			osync_change_unref(change);
 			osync_context_report_osyncwarning(ctx, error);
 			osync_error_unref(&error);
@@ -430,12 +433,11 @@ static void get_changes(void *userdata, OSyncPluginInfo *info, OSyncContext *ctx
 
 		osync_context_report_change(ctx, change);
 
-		osync_hashtable_update_hash(env->hashtable, osync_change_get_changetype(change), osync_change_get_uid(change), NULL);
+		osync_hashtable_update_change(env->hashtable, change);
 
 		osync_change_unref(change);
-		g_free(uids[i]);
 	}
-	g_free(uids);
+	osync_list_free(uids);
 
 	//Now we need to answer the call
 	osync_context_report_success(ctx);
@@ -533,7 +535,7 @@ static void commit_change(void *userdata, OSyncPluginInfo *info, OSyncContext *c
 	if(change_doc)
 		xmlFreeDoc(change_doc);
 
-	osync_hashtable_update_hash(env->hashtable, osync_change_get_changetype(change), osync_change_get_uid(change), hash);
+	osync_hashtable_update_change(env->hashtable, change);
 
 	//Answer the call
 	osync_context_report_success(ctx);
@@ -570,6 +572,9 @@ static void sync_done(void *userdata, OSyncPluginInfo *info, OSyncContext *ctx)
 		g_free(anchorpath);
 	}
 	
+	if (!osync_hashtable_save(env->hashtable, &error))
+		goto error;
+	
 	osync_context_report_success(ctx);
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	return;
@@ -586,7 +591,7 @@ static void disconnect(void *userdata, OSyncPluginInfo *info, OSyncContext *ctx)
 	OpieSinkEnv *env = osync_objtype_sink_get_userdata(sink);
 	
 	/* Close the hashtable */
-	osync_hashtable_free(env->hashtable);
+	osync_hashtable_unref(env->hashtable);
 	env->hashtable = NULL;
 
 	//Answer the call
@@ -595,9 +600,11 @@ static void disconnect(void *userdata, OSyncPluginInfo *info, OSyncContext *ctx)
 
 OpieSinkEnv *opie_sync_create_sink_env(OpiePluginEnv *env, OSyncPluginInfo *info, const char *objtype, const char *objformat, OPIE_OBJECT_TYPE opie_objtype, const char *remotefile, const char *listelement, const char *itemelement, OSyncError **error)
 {
-	OSyncObjTypeSink *sink = osync_objtype_sink_new(objtype, error);
-	if (!sink)
+	OSyncObjTypeSink *sink = osync_plugin_info_find_objtype(info, objtype);
+	if (!sink) {
+		osync_error_set(error, OSYNC_ERROR_MISCONFIGURATION, "Couldn't find Object Type \"%s\".", objtype);
 		return NULL;
+	}
 	
 	OpieSinkEnv *sink_env = osync_try_malloc0(sizeof(OpieSinkEnv), error);
 	if (!sink_env)
@@ -612,8 +619,6 @@ OpieSinkEnv *opie_sync_create_sink_env(OpiePluginEnv *env, OSyncPluginInfo *info
 	OSyncFormatEnv *formatenv = osync_plugin_info_get_format_env(info);
 	sink_env->objformat = osync_format_env_find_objformat(formatenv, objformat);
 
-	osync_objtype_sink_add_objformat(sink, objformat);
-
 	/* Every sink can have different functions ... */
 	OSyncObjTypeSinkFunctions functions;
 	memset(&functions, 0, sizeof(functions));
@@ -639,15 +644,7 @@ static void* opie_sync_initialize( OSyncPlugin *plugin, OSyncPluginInfo *info, O
 	if (!env)
 		goto error;
 	
-	const char *configdata = osync_plugin_info_get_config(info);
-	if (!configdata) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Unable to get config data.");
-		goto error_free_env;
-	}
-
-	osync_trace(TRACE_INTERNAL, "The config: %s", osync_plugin_info_get_config(info));
-	
-	if (!opie_sync_settings_parse(env, configdata, error))
+	if (!opie_sync_read_config(env, info, error))
 		goto error_free_env;
 	
 	env->backuppath = NULL;
@@ -724,10 +721,18 @@ static osync_bool opie_sync_discover(void *data, OSyncPluginInfo *info, OSyncErr
 	
 	OpiePluginEnv *env = (OpiePluginEnv *)data;
 	
-	osync_objtype_sink_set_available(env->contact_env->sink, TRUE);
+	int i, numobjs = osync_plugin_info_num_objtypes(info);
+	for (i = 0; i < numobjs; i++) {
+		OSyncObjTypeSink *sink = osync_plugin_info_nth_objtype(info, i);
+		g_assert(sink);
+
+		osync_objtype_sink_set_available(sink, TRUE);
+	}
+
+/*    osync_objtype_sink_set_available(env->contact_env->sink, TRUE);
 	osync_objtype_sink_set_available(env->todo_env->sink, TRUE);
 	osync_objtype_sink_set_available(env->event_env->sink, TRUE);
-	osync_objtype_sink_set_available(env->note_env->sink, TRUE);
+	osync_objtype_sink_set_available(env->note_env->sink, TRUE);*/
 	
 	OSyncVersion *version = osync_version_new(error);
 	osync_version_set_plugin(version, "opie-sync");
